/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.performance.consumer;

import java.util.logging.Level;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.objectweb.petals.component.common.util.SourceHelper;
import org.objectweb.petals.engine.performance.consumer.util.RandomMessageContent;

/**
 * @author alouis - EBM WebSourcing
 * 
 */
public class SendTask implements Runnable {

    /**
     * 
     */
    protected Sender sender;

    protected ComponentContext context;

    protected DeliveryChannel channel;

    protected MessageExchangeFactory factory;

    /**
     * Creates a new instance of SenderThread
     * 
     * @param monitoring
     * @param startSignal
     * @param doneSignal
     */
    public SendTask(Sender sender) {
        this.sender = sender;
        context = sender.context;
        channel = sender.channel;
        factory = channel.createExchangeFactory();
    }

    public void run() {
        // Send a message and accept a reponse message
        send();
        accept();
    }

    private void send() {
        try {
            // Construct message
            MessageExchange exchange = null;
            if (Sender.IN_ONLY.equals(sender.mep)) {
                exchange = factory.createInOnlyExchange();
            } else {
                exchange = factory.createInOutExchange();
            }

            if (sender.operation != null) {
                exchange.setOperation(new QName(sender.operation));
            }

            NormalizedMessage in = exchange.createMessage();

            // Fill the content of the message 
            Source content = null;
            if (sender.contentFileName != null) {
                synchronized (sender.data) {
                    content = SourceHelper.createSource(sender.data);
                }
            } else if (sender.randomSize) {
                content = SourceHelper.createSource(RandomMessageContent
                    .getRandomMessage());
            } else {
                synchronized (sender.data) {
                    content = SourceHelper.createSource(sender.data);
                }
            }
            in.setContent(content);

            // set properties
            if (sender.compression) {
                in.setProperty("org.objectweb.petals.transport.compress",
                    "true");
            }
            if (!sender.persistent) {
                exchange.setProperty(
                    "org.objectweb.petals.transport.nonpersistent", "true");
            }
            if (!sender.acknowledgment) {
                exchange.setProperty("org.objectweb.petals.messaging.noack",
                    "true");
            }

            exchange.setMessage(in, "IN");
            exchange.setEndpoint(sender.providerEndpoint);

            // Trace the messages if the tracking laps time has been reached
            String id = null;
            if (System.currentTimeMillis() > sender.lastTrackedMessageTime
                + sender.trackingLapsTime) {
                sender.lastTrackedMessageTime += sender.trackingLapsTime;
                exchange.setProperty("tracked", "true");
                id = exchange.getExchangeId();
                sender.beforeSendMarker.write(id + "\t"
                    + System.currentTimeMillis() + "\n");
            }

            // Send async or sync
            if (!sender.sync) {
                channel.send(exchange);
            } else {
                channel.sendSync(exchange, sender.syncTimeout);
            }

            // Trace the marked message
            if (id != null) {
                sender.afterSendMarker.write(id + "\t"
                    + System.currentTimeMillis() + "\n");
            }

            // Increment the counter
            sender.incSentMessageCount();

        } catch (Throwable e) {
            sender.incNotSentMessageCount();
            sender.logger.log(Level.SEVERE, e.getMessage(), e);
        }

    }

    private void accept() {
        try {
            MessageExchange receivedExchange = channel.accept(0);
            if (receivedExchange != null) {

                //Trace the message is necessary
                if (receivedExchange.getProperty("tracked") != null
                    && ((String) receivedExchange.getProperty("tracked"))
                        .equals("true")) {
                    String id = receivedExchange.getExchangeId();
                    sender.acceptMarker.write(id + "\t"
                        + System.currentTimeMillis() + "\n");
                }

                // incremente the matching counter
                if (ExchangeStatus.DONE.equals(receivedExchange.getStatus())) {
                    sender.incDoneExchangeCount();
                } else if (ExchangeStatus.ERROR.equals(receivedExchange
                    .getStatus())) {
                    sender.incErrorExchangeCount();
                } else {
                    if (receivedExchange.getFault() != null) {
                        sender.incErrorExchangeCount();
                    } else {
                        receivedExchange.setStatus(ExchangeStatus.DONE);
                        channel.send(receivedExchange);
                    }
                }
            }
        } catch (Throwable e) {
            sender.logger.log(Level.SEVERE, e.getMessage(), e);
        }
    }

}
